/**
 * 组件生命周期检查
 */
const { buildDocsMeta } = require('../utils');
const { PAGE_LIFECYCLES } = require('../constants');

const MESSAGE = 'WithPageLifecycles 必须在装饰器的最里层';
const LIFE_CYCLES_MESSAGE = '页面生命周期函数不要使用箭头函数形式，可能无法被调用';

/**
 * 禁止使用 data-set
 * @type {import('eslint').Rule.RuleModule}
 */
module.exports = {
  MESSAGE,
  LIFE_CYCLES_MESSAGE,
  meta: {
    type: 'problem',
    docs: buildDocsMeta(MESSAGE, 'with-page-lifecycles'),
  },

  create(context) {
    return {
      /**
       * @param {any} path
       */
      ClassDeclaration(path) {
        if (path.decorators && path.decorators.length) {
          const hocIndex = path.decorators.findIndex(
            i => i.expression.type === 'Identifier' && i.expression.name === 'WithPageLifecycles'
          );
          if (hocIndex !== -1 && hocIndex !== path.decorators.length - 1) {
            context.report({
              message: MESSAGE,
              node: path,
            });
          }
        }
      },
      /**
       * @param {any} path
       */
      ClassProperty(path) {
        if (path.key.type === 'Identifier' && PAGE_LIFECYCLES.includes(path.key.name)) {
          context.report({
            message: LIFE_CYCLES_MESSAGE,
            node: path,
          });
        }
      },
    };
  },
};
